//package junk;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.awt.geom.Point2D;
import java.awt.geom.Line2D;
import java.io.IOException;

//import java.util.Random;

public class GraphTools {

	/************************
	 * Static variables 
	 ************************/

	static String filename; // stores the last filename which we have used in
							// order to open a graph
	static SimpleGraph graph; // stores the last graph which was drawn

	static HashMap<Vertex, Point2D> randomVertexMap; // maps each vertex to a
														// geometric point
	static HashMap<Edge, Line2D> randomEdgeMap; // maps each vertex to a
												// geometric point
	static TreeMap<Edge, Line2D> sortedEdgeMap;

	static EdgeLengthComperator elc; // a comperator which compares the edges
										// based on their edge lengths.

	public static void drawRandomGeneratedGraph(SimpleGraph graph) {
		GraphGraphics.drawGraph(graph, randomVertexMap);
	}

	/********************************************************************
	 * Random 3-connected Planar Graph Generator: 1- Generate a random set V of
	 * points in unit square and make a vertex for each point 2- Let S be the
	 * set of all pairs (p,q) of points in V 3- Sort S into increasing order of
	 * distance between p and q 4- For all (p,q) in S in increasing order: if
	 * (p,q) does not cross any edge in E then add (p,q) to E
	 ********************************************************************/

	public static SimpleGraph generateRandom3ConnectedGraph(int numOfVertices) {

		// Random random = new Random(10);
		randomVertexMap = new HashMap<Vertex, Point2D>();
		randomEdgeMap = new HashMap<Edge, Line2D>();

		// creates a graph with "numOfVertices" nodes
		SimpleGraph graph = new SimpleGraph(numOfVertices);

		// Initialising hash map
		for (int i = 0; i < numOfVertices; i++) {

			// assigning a random point in a unit square to each vertex
			randomVertexMap.put(graph.getVertices().get(i), new Point2D.Double(
					Math.random(), Math.random()));
		}

		for (int i = 0; i < numOfVertices; i++) {
			for (int j = i + 1; j < numOfVertices; j++) {
				Edge newEdge = new Edge(graph.getVertices().get(i), graph
						.getVertices().get(j));
				graph.addEdge(newEdge);
				randomEdgeMap.put(
						newEdge,
						new Line2D.Double(randomVertexMap.get(graph
								.getVertices().get(i)), randomVertexMap
								.get(graph.getVertices().get(j))));
			}
		}

		elc = new EdgeLengthComperator(randomVertexMap);
		sortedEdgeMap = new TreeMap<Edge, Line2D>(elc);

		TreeMap<Edge, Line2D> tempEdgeMap = new TreeMap<Edge, Line2D>(elc);

		sortedEdgeMap.putAll(randomEdgeMap);
		tempEdgeMap.putAll(randomEdgeMap);

		for (Map.Entry<Edge, Line2D> entry1 : tempEdgeMap.entrySet()) {
			for (Map.Entry<Edge, Line2D> entry2 : tempEdgeMap.entrySet()) {
				if (entry2.getKey() == entry1.getKey())
					break;
				else if (entry1.getValue().intersectsLine(entry2.getValue())
						&& !twoEdgesShareAVertex(entry1.getKey(),
								entry2.getKey())
						&& graph.edgeExists(entry2.getKey())) {
					randomEdgeMap.remove(entry1.getKey());
					sortedEdgeMap.remove(entry1.getKey());
					graph.removeEdge(entry1.getKey());
				}
			}
		}

		return graph;
	}

	public static boolean twoEdgesShareAVertex(Edge e1, Edge e2) {
		return (e1.getV1() == e2.getV1() || e1.getV1() == e2.getV2()
				|| e1.getV2() == e2.getV1() || e1.getV2() == e2.getV2());
	}

	/*******************
	 * Other Methods 
	 *******************/

	// This method returns a 3-cycle in the given graph if any.
	public static Vertex[] findA3CycleIn(SimpleGraph graph) {
		Vertex[] cycle = new Vertex[3];
		Edge edge = graph.getEdges().get(0);
		cycle[0] = edge.getV1();
		cycle[1] = edge.getV2();
		for (Vertex v : graph.getVertices())
			if (graph.edgeExists(new Edge(edge.getV1(), v))
					&& graph.edgeExists(new Edge(edge.getV2(), v))) {
				cycle[2] = v;
				break;
			}
		return cycle;
	}

	// This method finds 3 vertices that make a cycle and put them at the head
	// of the list.
	public static int choose3FixedVertices(SimpleGraph graph) {
		Edge edge = graph.getEdges().get(0);
		Vertex v1 = edge.getV1();
		Vertex v2 = edge.getV2();
		Vertex v3 = new Vertex();

		// find v3 such that it makes a 3-cycle with v1 and v2
		for (int i = 0; i < graph.getVertices().size(); i++) {
			if (graph.edgeExists(new Edge(v1, graph.getVertices().get(i)))
					&& graph.edgeExists(new Edge(v2, graph.getVertices().get(i)))) {
				v3 = graph.getVertices().get(i);
				break;
			}
		}

		// Insert v1,v2 and v3 as the first, second and third vertices in the
		// list of vertices.
		graph.getVertices().remove(v1);
		graph.getVertices().add(0, v1);

		graph.getVertices().remove(v2);
		graph.getVertices().add(0, v2);

		graph.getVertices().remove(v3);
		graph.getVertices().add(0, v3);

		return 3;
	}

	// This method redraws the simple graph with the filename which was
	// previously stored in GraphTools class
	public static void redraw() {
		graph = new SimpleGraph(filename);
		graph.drawTutteWeightedGraph();
	}

	// This method redraws the simple graph with the filename as an argument
	public static void redraw(String filename) {
		graph = new SimpleGraph(filename);
		graph.drawTutteWeightedGraph();
		try {
			Runtime.getRuntime().exec("notepad " + GraphTools.filename);
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/******************
	 * Convex Hull
	 ******************/

	public static int convexHull(ArrayList<Vertex> vertices,
			HashMap<Vertex, Point2D> map) {
		ArrayList<Vertex> xSorted = (ArrayList<Vertex>) vertices.clone();
		Collections.sort(xSorted, new XCompare(map));

		int n = xSorted.size();

		Vertex[] lUpper = new Vertex[n];

		lUpper[0] = xSorted.get(0);
		lUpper[1] = xSorted.get(1);

		int lUpperSize = 2;

		for (int i = 2; i < n; i++) {
			lUpper[lUpperSize] = xSorted.get(i);
			lUpperSize++;

			while (lUpperSize > 2
					&& !rightTurn(map.get(lUpper[lUpperSize - 3]),
							map.get(lUpper[lUpperSize - 2]),
							map.get(lUpper[lUpperSize - 1]))) {
				// Remove the middle point of the three last
				lUpper[lUpperSize - 2] = lUpper[lUpperSize - 1];
				lUpperSize--;
			}
		}

		Vertex[] lLower = new Vertex[n];

		lLower[0] = xSorted.get(n - 1);
		lLower[1] = xSorted.get(n - 2);

		int lLowerSize = 2;

		for (int i = n - 3; i >= 0; i--) {
			lLower[lLowerSize] = xSorted.get(i);
			lLowerSize++;

			while (lLowerSize > 2
					&& !rightTurn(map.get(lLower[lLowerSize - 3]),
							map.get(lLower[lLowerSize - 2]),
							map.get(lLower[lLowerSize - 1]))) {
				// Remove the middle point of the three last
				lLower[lLowerSize - 2] = lLower[lLowerSize - 1];
				lLowerSize--;
			}
		}

		ArrayList<Vertex> result = new ArrayList<Vertex>();

		for (int i = 0; i < lUpperSize; i++) {
			result.add(lUpper[i]);
		}

		for (int i = 1; i < lLowerSize - 1; i++) {
			result.add(lLower[i]);
		}

		for (int i = 0; i < result.size(); i++) {
			vertices.remove(result.get(i));
			vertices.add(0, result.get(i));
		}

		return result.size();
	}

	private static boolean rightTurn(Point2D a, Point2D b, Point2D c) {
		return (b.getX() - a.getX()) * (c.getY() - a.getY())
				- (b.getY() - a.getY()) * (c.getX() - a.getX()) > 0;
	}

	/*******************
	 * Main Method
	 *******************/

	// Main method: this method will create a random graph and draw it using
	// Tutte's algorithm
	public static void main(String[] args) {

		int numOfVertices = 5 + (int) (Math.random() * 20);
		graph = generateRandom3ConnectedGraph(numOfVertices);

		int numOfFixedVertices;
		if (Math.random() < 0.2)
			numOfFixedVertices = choose3FixedVertices(graph);
		else
			numOfFixedVertices = convexHull(graph.getVertices(),
					randomVertexMap);

		graph.setNumOfFixedVertices(numOfFixedVertices);

		// drawRandomGeneratedGraph(graph);
		graph.drawTutteWeightedGraph();
	}
}
