import java.awt.geom.Point2D;
import java.util.HashMap;

public class TutteDrawing {

	private SimpleGraph graph;

	public TutteDrawing(SimpleGraph graph) {
		this.graph = graph;
	}

	/******************************
	 * 
	 * Tutte's Drawing Algorithm
	 * 
	 ******************************/

	// This method receives a graph and the number of fixed vertices as input
	// and maps the vertices to geometric points on plane
	public HashMap<Vertex, Point2D> calculate() {
		
		int n= graph.n();  // total number of vertices
		int numFixed= graph.getNumOfFixedVertices();  // number of fixed vertices
		int[][] g = graph.adjacencyMatrix();  // adjacency matrix
		HashMap<Vertex, Point2D> tutteMap = new HashMap<Vertex, Point2D>();  // maps each vertex to a geometric point

		// Initialising hash map
		for (int i = 0; i < n; i++) {
			tutteMap.put(graph.getVertices().get(i), new Point2D.Double());
		}

		for (int i = numFixed; i < n; i++) {
			double theta = 2.0 * Math.PI * Math.random();
			tutteMap.get(graph.getVertices().get(i)).setLocation(
					0.5 + 0.5 * Math.cos(theta * i),
					0.5 + 0.5 * Math.sin(theta * i));
		}
		double theta = 2.0 * Math.PI / ((double) numFixed);
		for (int i = 0; i < numFixed; i++) {
			tutteMap.get(graph.getVertices().get(i)).setLocation(
					0.5 + 0.5 * Math.cos(theta * i),
					0.5 + 0.5 * Math.sin(theta * i));
		}

		int k = 0;
		int staticCountLimit = 200 * n;
		int staticCount = 0;
		double epsilon = 0.0001;
		while ((k < 10000000) && (staticCount < staticCountLimit)) {
			for (int i = numFixed; i < n; i++) {

				double sumx = 0.0;
				double sumy = 0.0;
				double d = 0.0;
				for (int j = 0; j < n; j++) {
					if (g[i][j] > 0) {
						sumx = sumx
								+ g[i][j]
								* tutteMap.get(graph.getVertices().get(j))
										.getX();
						sumy = sumy
								+ g[i][j]
								* tutteMap.get(graph.getVertices().get(j))
										.getY();
						d = d + g[i][j];
					}
				}
				double newX = sumx / d;
				double newY = sumy / d;
				if ((tutteMap.get(graph.getVertices().get(i)).getX() - newX)
						* (tutteMap.get(graph.getVertices().get(i)).getX() - newX)
						+ (tutteMap.get(graph.getVertices().get(i)).getY() - newY)
						* (tutteMap.get(graph.getVertices().get(i)).getY() - newY) < epsilon) {
					staticCount++;
				} else {
					staticCount = 0;
				}

				tutteMap.get(graph.getVertices().get(i))
						.setLocation(newX, newY);
				k++;
			}
		}
		return tutteMap;
	}

}
